#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Run pylint in a docker
"""
import argparse
import logging
import os
import sys
from typing import Dict, List

from docker.types import Mount

from .config import ODOO_7, ODOO_8, ODOO_10, ODOO_11, ODOO_13, ODOO_15, Config
from .docker_build import add_build_options, build_local_image, get_build_options
from .docker_client import DockerClient, anthem_mounts, modules_mount
from .parsing import apply, basic_parser

__version__ = "1.1.0"
__date__ = "2022-01-26"
__updated__ = "2022-04-20"

_logger = logging.getLogger(__name__)

_PYLINT_DEST = "pylint"
"""Name of destination variable used for pylint in parsing"""


def __parser():
    program_version_message = f"%(prog)s {__version__} ({__updated__})"
    program_short_description = __doc__.split(".", maxsplit=1)[0]
    program_license = f"""{program_short_description}

      Created by Vincent Hatakeyama on {__date__}.
      Copyright 2022 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """
    parser = basic_parser(program_license, program_version_message)
    return parser


def parser_add_pylint_group(parser: argparse.ArgumentParser):
    """Add pylint option to the given parser.
    Defaults to not running pylint."""
    parser.add_argument(
        "--pylint",
        help="Run pylint on given python module (for example odoo.addons.base)",
        dest=_PYLINT_DEST,
        default=[],
        nargs="+",
    )


def apply_pylint(namespace: argparse.Namespace):
    """Run pylint if the option was set."""
    if _PYLINT_DEST in namespace and getattr(namespace, _PYLINT_DEST):
        return pylint(getattr(namespace, _PYLINT_DEST), get_build_options(namespace))
    return 0


def main(argv=None):
    """Copy modules for a build, callable version that parses arguments"""
    parser = __parser()
    parser_add_pylint_group(parser)
    add_build_options(parser)
    nmspc = parser.parse_args(argv)
    apply(nmspc)
    return apply_pylint(nmspc)


def pylint(modules, build_options):
    """Run pylint"""
    project_path = os.path.realpath(".")

    # always use project image
    config = Config()

    odoo_type = config.odoo_type

    mounts: List[Mount] = []
    environment = {}
    target_source_dict: Dict[str, str] = {}

    # odoo configuration is not read so put the modules in the path
    if odoo_type in (ODOO_10, ODOO_11, ODOO_13, ODOO_15):
        if odoo_type == ODOO_10:
            pythons = ("python2.7",)
        elif odoo_type == ODOO_11:
            pythons = ("python3.5", "python3.6")
        elif odoo_type == ODOO_13:
            pythons = ("python3.8",)
        elif odoo_type == ODOO_15:
            pythons = ("python3.9",)
        for python in pythons:
            result = modules_mount(
                config.modules,
                project_path,
                f"/usr/local/lib/{python}/dist-packages/odoo/addons",
            )
            mounts.extend(result[0])
            target_source_dict.update(result[1])
    elif odoo_type in (ODOO_7, ODOO_8):
        mounts, target_source_dict = modules_mount(
            config.modules, project_path, "/opt/odoo/sources/odoo/addons"
        )

    # Add a volume to store cache
    cache_volume_name = "pylint-cache"
    DockerClient.create_volume(cache_volume_name)
    cache_target_path = "/var/cache/pylint"
    mounts.append(Mount(cache_target_path, cache_volume_name))
    environment["PYLINTHOME"] = cache_target_path

    mounts.extend(anthem_mounts(project_path, pythons))

    build_local_image(build_options)

    _logger.info("Running pylint")
    return DockerClient.run(
        config.local_image,
        config.local_tag,
        {
            "entrypoint": "pylint",
            "command": modules + ["--load-plugins=pylint_odoo"],
            # needed to write in the cache
            "user": "root",
            "mounts": mounts,
            "environment": environment,
        },
        False,
        target_source_dict,
    )


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
